@@ -66,17 +66,12 @@ void
 notification_plugin_popup_menu (NotificationPlugin *notification_plugin)
 {
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (notification_plugin->button), TRUE);
-#if GTK_CHECK_VERSION (3, 22, 0)
   gtk_menu_popup_at_widget (GTK_MENU (notification_plugin->menu),
                             notification_plugin->button,
                             xfce_panel_plugin_get_orientation (notification_plugin->plugin) == GTK_ORIENTATION_VERTICAL
                             ? GDK_GRAVITY_NORTH_EAST : GDK_GRAVITY_SOUTH_WEST,
                             GDK_GRAVITY_NORTH_WEST,
                             NULL);
-#else
-  gtk_menu_popup (GTK_MENU (notification_plugin->menu), NULL, NULL,
-                  xfce_panel_plugin_position_menu, notification_plugin, 0, 0);
-#endif
   xfce_panel_plugin_register_menu (notification_plugin->plugin,
                                    GTK_MENU (notification_plugin->menu));
 }
@@ -111,6 +106,9 @@ cb_menu_deactivate (GtkMenuShell *menu,
                     NotificationPlugin *notification_plugin)
 {
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (notification_plugin->button), FALSE);
+  gtk_widget_set_visible(notification_plugin->button,
+                         !notification_plugin->hide_on_read
+                         || notification_plugin->new_notifications);
 }
 
 
@@ -143,22 +141,46 @@ cb_menu_size_allocate (GtkWidget          *menu,
 
 
 
+static void
+cb_hide_on_read_changed(XfconfChannel *channel,
+                        const gchar *property_name,
+                        const GValue *value,
+                        NotificationPlugin *notification_plugin)
+{
+  if (G_VALUE_HOLDS_BOOLEAN(value)) {
+    notification_plugin->hide_on_read = g_value_get_boolean(value);
+    gtk_widget_set_visible(notification_plugin->button,
+                           !notification_plugin->hide_on_read
+                           || notification_plugin->new_notifications
+                           || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(notification_plugin->button)));
+  }
+}
+
+
+
 void
 notification_plugin_update_icon (NotificationPlugin *notification_plugin,
                                  gboolean state)
 {
-  if (state && !notification_plugin->new_notifications)
-    gtk_image_set_from_icon_name (GTK_IMAGE (notification_plugin->image),
-                                  "notification-disabled-symbolic", GTK_ICON_SIZE_MENU);
-  else if (!state && !notification_plugin->new_notifications)
-    gtk_image_set_from_icon_name (GTK_IMAGE (notification_plugin->image),
-                                  "notification-symbolic", GTK_ICON_SIZE_MENU);
-  else if (state && notification_plugin->new_notifications)
-    gtk_image_set_from_icon_name (GTK_IMAGE (notification_plugin->image),
-                                  "notification-disabled-new-symbolic", GTK_ICON_SIZE_MENU);
-  else if (!state && notification_plugin->new_notifications)
-    gtk_image_set_from_icon_name (GTK_IMAGE (notification_plugin->image),
-                                  "notification-new-symbolic", GTK_ICON_SIZE_MENU);
+  const gchar *icon_name = NULL;
+
+  if (state && !notification_plugin->new_notifications) {
+    icon_name = "notification-disabled-symbolic";
+  } else if (!state && !notification_plugin->new_notifications) {
+    icon_name = "notification-symbolic";
+  } else if (state && notification_plugin->new_notifications) {
+    icon_name = "notification-disabled-new-symbolic";
+  } else if (!state && notification_plugin->new_notifications) {
+    icon_name = "notification-new-symbolic";
+  }
+
+  if (G_LIKELY(icon_name != NULL)) {
+    gtk_image_set_from_icon_name(GTK_IMAGE(notification_plugin->image), icon_name, GTK_ICON_SIZE_MENU);
+  }
+  gtk_widget_set_visible(notification_plugin->button,
+                         !notification_plugin->hide_on_read
+                         || notification_plugin->new_notifications
+                         || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(notification_plugin->button)));
 }
 
 
@@ -205,8 +227,6 @@ static NotificationPlugin *
 notification_plugin_new (XfcePanelPlugin *panel_plugin)
 {
   NotificationPlugin    *notification_plugin;
-  GFile                 *log_file;
-  GFileMonitor          *log_file_monitor;
   gchar                 *notify_log_path = NULL;
   gboolean               state;
 
@@ -218,6 +238,10 @@ notification_plugin_new (XfcePanelPlugin *panel_plugin)
   xfconf_init (NULL);
   notification_plugin->channel = xfconf_channel_new ("xfce4-notifyd");
 
+  notification_plugin->hide_on_read = xfconf_channel_get_bool(notification_plugin->channel, SETTING_HIDE_ON_READ, FALSE);
+  g_signal_connect(notification_plugin->channel, "property-changed::" SETTING_HIDE_ON_READ,
+                   G_CALLBACK(cb_hide_on_read_changed), notification_plugin);
+
   /* As the plugin is starting up we presume there are no new notifications */
   notification_plugin->new_notifications = FALSE;
 
@@ -227,13 +251,13 @@ notification_plugin_new (XfcePanelPlugin *panel_plugin)
   gtk_widget_set_tooltip_text (GTK_WIDGET (notification_plugin->button), _("Notifications"));
   notification_plugin->image = gtk_image_new ();
   state = xfconf_channel_get_bool (notification_plugin->channel, "/do-not-disturb", FALSE);
-  notification_plugin_update_icon (notification_plugin, state);
 
   gtk_container_add (GTK_CONTAINER (notification_plugin->button), notification_plugin->image);
-  gtk_container_add (GTK_CONTAINER (panel_plugin), notification_plugin->button);
   gtk_widget_show_all (GTK_WIDGET (notification_plugin->button));
   gtk_widget_set_name (GTK_WIDGET (notification_plugin->button), "xfce4-notification-plugin");
 
+  notification_plugin_update_icon(notification_plugin, state);
+
   /* Create the menu */
   notification_plugin->menu = notification_plugin_menu_new (notification_plugin);
   gtk_menu_attach_to_widget (GTK_MENU (notification_plugin->menu), notification_plugin->button, NULL);
@@ -248,10 +272,15 @@ notification_plugin_new (XfcePanelPlugin *panel_plugin)
 
   /* Start monitoring the log file for changes */
   notify_log_path = xfce_resource_lookup (XFCE_RESOURCE_CACHE, XFCE_NOTIFY_LOG_FILE);
-  log_file = g_file_new_for_path (notify_log_path);
-  log_file_monitor = g_file_monitor_file (log_file, G_FILE_MONITOR_NONE, NULL, NULL);
-  g_signal_connect (log_file_monitor, "changed",
-                    G_CALLBACK (notification_plugin_log_file_changed), notification_plugin);
+  if (notify_log_path) {
+    notification_plugin->log_file = g_file_new_for_path (notify_log_path);
+    notification_plugin->log_file_monitor = g_file_monitor_file (notification_plugin->log_file,
+                                                                 G_FILE_MONITOR_NONE, NULL, NULL);
+    if (notification_plugin->log_file_monitor != NULL)
+      g_signal_connect (notification_plugin->log_file_monitor, "changed",
+                        G_CALLBACK (notification_plugin_log_file_changed), notification_plugin);
+    g_free (notify_log_path);
+  }
 
   /* Start monitoring the "do not disturb" setting in xfconf */
   g_signal_connect (G_OBJECT (notification_plugin->channel), "property-changed::" "/do-not-disturb",
@@ -268,6 +297,13 @@ notification_plugin_free (XfcePanelPlugin *plugin,
 {
   GtkWidget *dialog;
 
+  /* detach from the file monitor */
+  if (notification_plugin->log_file_monitor != NULL) {
+    g_file_monitor_cancel (notification_plugin->log_file_monitor);
+    g_object_unref (notification_plugin->log_file_monitor);
+  }
+  g_object_unref (notification_plugin->log_file);
+
   /* check if the dialog is still open. if so, destroy it */
   dialog = g_object_get_data (G_OBJECT (plugin), "dialog");
   if (G_UNLIKELY (dialog != NULL))
