@@ -25,13 +25,12 @@
 #define XFCE_TYPE_NOTIFY_DAEMON     (xfce_notify_daemon_get_type())
 #define XFCE_NOTIFY_DAEMON(obj)     (G_TYPE_CHECK_INSTANCE_CAST((obj), XFCE_TYPE_NOTIFY_DAEMON, XfceNotifyDaemon))
 #define XFCE_IS_NOTIFY_DAEMON(obj)  (G_TYPE_CHECK_INSTANCE_TYPE((obj), XFCE_TYPE_NOTIFY_DAEMON))
-#define NOTIFICATIONS_SPEC_VERSION "1.0"
 
 G_BEGIN_DECLS
 
 typedef struct _XfceNotifyDaemon  XfceNotifyDaemon;
 
-GType xfce_notify_daemon_get_type() G_GNUC_CONST;
+GType xfce_notify_daemon_get_type(void) G_GNUC_CONST;
 
 XfceNotifyDaemon *xfce_notify_daemon_new_unique(GError **error);
 
