@@ -31,6 +31,17 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
+#ifdef ENABLE_WAYLAND
+#include <gdk/gdkwayland.h>
+#include <gtk-layer-shell.h>
+#endif
+
+#ifdef ENABLE_SOUND
+#include <canberra-gtk.h>
+#endif
+
+#include <common/xfce-notify-common.h>
+
 #include "xfce-notify-window.h"
 #include "xfce-notify-enum-types.h"
 
@@ -49,6 +60,8 @@ struct _XfceNotifyWindow
 {
     GtkWindow parent;
 
+    guint id;
+
     GdkRectangle geometry;
     gint last_monitor;
 
@@ -62,7 +75,7 @@ struct _XfceNotifyWindow
     guint32 icon_only:1,
             has_summary_text:1,
             has_body_text:1,
-            has_actions;
+            has_actions:1;
 
     GtkWidget *icon_box;
     GtkWidget *icon;
@@ -80,6 +93,10 @@ struct _XfceNotifyWindow
     gboolean do_fadeout;
     gboolean do_slideout;
     GtkCornerType notify_location;
+
+#ifdef ENABLE_SOUND
+    ca_proplist *sound_props;
+#endif
 };
 
 typedef struct
@@ -110,6 +127,8 @@ static gboolean xfce_notify_window_enter_leave(GtkWidget *widget,
                                                GdkEventCrossing *evt);
 static gboolean xfce_notify_window_button_release(GtkWidget *widget,
                                                   GdkEventButton *evt);
+static gboolean xfce_notify_window_motion_notify(GtkWidget *widget,
+                                                 GdkEventMotion *evt);
 static gboolean xfce_notify_window_configure_event(GtkWidget *widget,
                                                    GdkEventConfigure *evt);
 static gboolean xfce_notify_window_expire_timeout(gpointer data);
@@ -138,6 +157,7 @@ xfce_notify_window_class_init(XfceNotifyWindowClass *klass)
     widget_class->draw = xfce_notify_window_draw;
     widget_class->enter_notify_event = xfce_notify_window_enter_leave;
     widget_class->leave_notify_event = xfce_notify_window_enter_leave;
+    widget_class->motion_notify_event =xfce_notify_window_motion_notify;
     widget_class->button_release_event = xfce_notify_window_button_release;
     widget_class->configure_event = xfce_notify_window_configure_event;
 
@@ -179,16 +199,24 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     gint screen_width;
     gdouble padding = DEFAULT_PADDING;
     GtkCssProvider *provider;
-#if GTK_CHECK_VERSION (3, 22, 0)
     GdkMonitor *monitor;
     GdkRectangle geometry;
-#endif
 
     window->expire_timeout = DEFAULT_EXPIRE_TIMEOUT;
     window->normal_opacity = DEFAULT_NORMAL_OPACITY;
     window->do_fadeout = DEFAULT_DO_FADEOUT;
     window->do_slideout = DEFAULT_DO_SLIDEOUT;
 
+#ifdef ENABLE_WAYLAND
+    if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+        gtk_layer_init_for_window(GTK_WINDOW(window));
+        gtk_layer_set_layer(GTK_WINDOW(window), GTK_LAYER_SHELL_LAYER_TOP);
+        gtk_layer_set_namespace(GTK_WINDOW(window), "notification");
+        gtk_layer_set_anchor(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP, TRUE);
+        gtk_layer_set_anchor(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT, TRUE);
+    }
+#endif
+
     gtk_widget_set_name (GTK_WIDGET(window), "XfceNotifyWindow");
     gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
     gtk_window_stick(GTK_WINDOW(window));
@@ -215,17 +243,13 @@ xfce_notify_window_init(XfceNotifyWindow *window)
         gtk_widget_set_visual (GTK_WIDGET(window), visual);
     }
 
-    /* Use the screen width to get a maximum width for the notification bubble.
+    /* Use the monitor width to get a maximum width for the notification bubble.
        This assumes that a character is 10px wide and we want a third of the
-       screen as maximum width. */
-#if GTK_CHECK_VERSION (3, 22, 0)
+       monitor as maximum width. */
     monitor = gdk_display_get_monitor_at_window (gtk_widget_get_display (GTK_WIDGET (window)),
                                                  gdk_screen_get_root_window (screen));
     gdk_monitor_get_geometry (monitor, &geometry);
     screen_width = geometry.width / 30;
-#else
-    screen_width = gdk_screen_get_width (screen) / 30;
-#endif
 
     gtk_widget_style_get(GTK_WIDGET(window),
                          "padding", &padding,
@@ -265,9 +289,7 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     gtk_label_set_line_wrap_mode (GTK_LABEL (window->summary), PANGO_WRAP_WORD_CHAR);
     gtk_label_set_lines (GTK_LABEL (window->summary), 1);
     gtk_widget_set_halign (window->summary, GTK_ALIGN_FILL);
-#if GTK_CHECK_VERSION (3, 16, 0)
     gtk_label_set_xalign (GTK_LABEL(window->summary), 0);
-#endif
     gtk_widget_set_valign (window->summary, GTK_ALIGN_BASELINE);
     gtk_box_pack_start(GTK_BOX(vbox), window->summary, TRUE, TRUE, 0);
 
@@ -279,9 +301,7 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     gtk_label_set_line_wrap_mode (GTK_LABEL (window->body), PANGO_WRAP_WORD_CHAR);
     gtk_label_set_lines (GTK_LABEL (window->body), 6);
     gtk_widget_set_halign (window->body, GTK_ALIGN_FILL);
-#if GTK_CHECK_VERSION (3, 16, 0)
     gtk_label_set_xalign (GTK_LABEL(window->body), 0);
-#endif
     gtk_widget_set_valign (window->body, GTK_ALIGN_BASELINE);
     gtk_box_pack_start(GTK_BOX(vbox), window->body, TRUE, TRUE, 0);
 
@@ -332,6 +352,14 @@ xfce_notify_window_start_expiration(XfceNotifyWindow *window)
 static void
 xfce_notify_window_finalize(GObject *object)
 {
+#ifdef ENABLE_SOUND
+    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(object);
+
+    if (window->sound_props != NULL) {
+        ca_proplist_destroy(window->sound_props);
+    }
+#endif
+
     G_OBJECT_CLASS(xfce_notify_window_parent_class)->finalize(object);
 }
 
@@ -340,12 +368,35 @@ xfce_notify_window_realize(GtkWidget *widget)
 {
     XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
 
+#ifdef ENABLE_WAYLAND
+    if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+        gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT, window->geometry.x);
+        gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP, window->geometry.y);
+    }
+#endif
+
     GTK_WIDGET_CLASS(xfce_notify_window_parent_class)->realize(widget);
 
     gdk_window_set_type_hint(gtk_widget_get_window(widget),
                              GDK_WINDOW_TYPE_HINT_NOTIFICATION);
     gdk_window_set_override_redirect(gtk_widget_get_window(widget), TRUE);
     xfce_notify_window_start_expiration(window);
+
+#ifdef ENABLE_SOUND
+    if (window->sound_props != NULL) {
+        ca_context *ctx;
+        const gchar *summary;
+
+        summary = gtk_label_get_text(GTK_LABEL(window->summary));
+        if (summary != NULL) {
+            ca_proplist_sets(window->sound_props, CA_PROP_EVENT_DESCRIPTION, summary);
+        }
+        ca_gtk_proplist_set_for_widget(window->sound_props, widget);
+
+        ctx = ca_gtk_context_get_for_screen(gtk_widget_get_screen(widget));
+        ca_context_play_full(ctx, window->id, window->sound_props, NULL, NULL);
+    }
+#endif
 }
 
 static void
@@ -417,29 +468,52 @@ xfce_notify_window_enter_leave(GtkWidget *widget,
 {
     XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
 
-    if(evt->type == GDK_ENTER_NOTIFY) {
-        if(window->expire_timeout) {
-            if(window->expire_id) {
-                g_source_remove(window->expire_id);
-                window->expire_id = 0;
-            }
-            if(window->fade_id) {
-                g_source_remove(window->fade_id);
-                window->fade_id = 0;
-                /* reset the sliding-out window to its original position */
-                if (window->do_slideout)
+    if (evt->type == GDK_LEAVE_NOTIFY && evt->detail != GDK_NOTIFY_INFERIOR) {
+        if (window->expire_id == 0) {
+            xfce_notify_window_start_expiration(window);
+        }
+        window->mouse_hover = FALSE;
+        gtk_widget_queue_draw(widget);
+    }
+
+    return FALSE;
+}
+
+static gboolean
+xfce_notify_window_motion_notify(GtkWidget *widget,
+                                 GdkEventMotion *evt)
+{
+    XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
+
+    if (window->expire_timeout != 0) {
+        if (window->expire_id != 0) {
+            g_source_remove(window->expire_id);
+            window->expire_id = 0;
+        }
+
+        if (window->fade_id != 0) {
+            g_source_remove(window->fade_id);
+            window->fade_id = 0;
+
+            /* reset the sliding-out window to its original position */
+            if (window->do_slideout) {
+#ifdef ENABLE_WAYLAND
+                if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+                    gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT, window->original_x);
+                    gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP, window->original_y);
+                } else
+#endif
+                {
                     gtk_window_move (GTK_WINDOW (window), window->original_x, window->original_y);
+                }
             }
         }
+    }
+
+    if (!window->mouse_hover) {
         gtk_widget_set_opacity(GTK_WIDGET(widget), 1.0);
         window->mouse_hover = TRUE;
         gtk_widget_queue_draw(widget);
-    } else if(evt->type == GDK_LEAVE_NOTIFY
-              && evt->detail != GDK_NOTIFY_INFERIOR)
-    {
-        xfce_notify_window_start_expiration(window);
-        window->mouse_hover = FALSE;
-        gtk_widget_queue_draw(widget);
     }
 
     return FALSE;
@@ -488,7 +562,15 @@ xfce_notify_window_expire_timeout(gpointer data)
     if(fade_transparent && window->do_fadeout) {
         /* remember the original position of the window before we slide it out */
         if (window->do_slideout) {
-            gtk_window_get_position (GTK_WINDOW (window), &window->original_x, &window->original_y);
+#ifdef ENABLE_WAYLAND
+            if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+                window->original_x = gtk_layer_get_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT);
+                window->original_y = gtk_layer_get_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP);
+            } else
+#endif
+            {
+                gtk_window_get_position (GTK_WINDOW (window), &window->original_x, &window->original_y);
+            }
             animation_timeout = FADE_CHANGE_TIMEOUT / 2;
         }
         else
@@ -516,7 +598,16 @@ xfce_notify_window_fade_timeout(gpointer data)
 
     /* slide out animation */
     if (window->do_slideout) {
-        gtk_window_get_position (GTK_WINDOW (window), &x, &y);
+#ifdef ENABLE_WAYLAND
+        if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+            x = gtk_layer_get_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT);
+            y = gtk_layer_get_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP);
+        } else
+#endif
+        {
+            gtk_window_get_position (GTK_WINDOW (window), &x, &y);
+        }
+
         if (window->notify_location == GTK_CORNER_TOP_RIGHT ||
             window->notify_location == GTK_CORNER_BOTTOM_RIGHT)
             x = x + 10;
@@ -525,7 +616,17 @@ xfce_notify_window_fade_timeout(gpointer data)
             x = x - 10;
         else
             g_warning("Invalid notify location: %d", window->notify_location);
-        gtk_window_move (GTK_WINDOW (window), x, y);
+
+
+#ifdef ENABLE_WAYLAND
+        if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+            gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT, x);
+            gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP, y);
+        } else
+#endif
+        {
+            gtk_window_move (GTK_WINDOW (window), x, y);
+        }
     }
 
     /* fade-out animation */
@@ -569,7 +670,7 @@ xfce_notify_window_button_clicked(GtkWidget *widget,
 GtkWidget *
 xfce_notify_window_new(void)
 {
-    return xfce_notify_window_new_with_actions(NULL, NULL, NULL, -1, NULL, NULL);
+    return xfce_notify_window_new_with_actions(NULL, NULL, NULL, -1, NULL, FALSE, NULL);
 }
 
 GtkWidget *
@@ -582,6 +683,7 @@ xfce_notify_window_new_full(const gchar *summary,
                                                icon_name,
                                                expire_timeout,
                                                NULL,
+                                               FALSE,
                                                NULL);
 }
 
@@ -591,6 +693,7 @@ xfce_notify_window_new_with_actions(const gchar *summary,
                                     const gchar *icon_name,
                                     gint expire_timeout,
                                     const gchar **actions,
+                                    gboolean actions_are_icon_names,
                                     GtkCssProvider *css_provider)
 {
     XfceNotifyWindow *window;
@@ -605,11 +708,26 @@ xfce_notify_window_new_with_actions(const gchar *summary,
     xfce_notify_window_set_body(window, body);
     xfce_notify_window_set_icon_name(window, icon_name);
     xfce_notify_window_set_expire_timeout(window, expire_timeout);
-    xfce_notify_window_set_actions(window, actions, css_provider);
+    xfce_notify_window_set_actions(window, actions, actions_are_icon_names, css_provider);
 
     return GTK_WIDGET(window);
 }
 
+void
+xfce_notify_window_set_id(XfceNotifyWindow *window,
+                          guint id)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+    window->id = id;
+}
+
+guint
+xfce_notify_window_get_id(XfceNotifyWindow *window)
+{
+    g_return_val_if_fail(XFCE_IS_NOTIFY_WINDOW(window), 0);
+    return window->id;
+}
+
 void
 xfce_notify_window_set_summary(XfceNotifyWindow *window,
                                const gchar *summary)
@@ -636,16 +754,10 @@ xfce_notify_window_set_body(XfceNotifyWindow *window,
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
 
     if(body && *body) {
-        /* Try to set the body with markup and in case this fails (empty label)
-           fall back to escaping the whole string and showing it plainly.
-           This equals pango_parse_markup extended by checking for valid hyperlinks
-           (which is not supported by pango). */
-        gtk_label_set_markup (GTK_LABEL (window->body), body);
-        if (g_strcmp0 (gtk_label_get_text(GTK_LABEL (window->body)), "") == 0 ) {
-            gchar *tmp;
-            tmp = g_markup_escape_text (body, -1);
-            gtk_label_set_text (GTK_LABEL (window->body), body);
-            g_free (tmp);
+        if (xfce_notify_is_markup_valid(body)) {
+            gtk_label_set_markup (GTK_LABEL (window->body), body);
+        } else {
+            gtk_label_set_text(GTK_LABEL(window->body), body);
         }
         gtk_widget_show(window->body);
         window->has_body_text = TRUE;
@@ -664,6 +776,13 @@ xfce_notify_window_set_geometry(XfceNotifyWindow *window,
                                 GdkRectangle rectangle)
 {
     window->geometry = rectangle;
+
+#ifdef ENABLE_WAYLAND
+    if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+        gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT, window->geometry.x);
+        gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP, window->geometry.y);
+    }
+#endif
 }
 
 GdkRectangle *
@@ -689,96 +808,91 @@ void
 xfce_notify_window_set_icon_name (XfceNotifyWindow *window,
                                   const gchar *icon_name)
 {
-    gboolean icon_set = FALSE;
+    GIcon *icon = NULL;
 
     g_return_if_fail (XFCE_IS_NOTIFY_WINDOW (window));
 
     if (icon_name && *icon_name) {
-        gint w, h;
-        GdkPixbuf *pix = NULL;
-        GIcon *icon;
+        gboolean is_absolute, is_uri;
 
-        gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, &w, &h);
+        is_absolute = g_path_is_absolute (icon_name);
+        is_uri = g_str_has_prefix(icon_name, "file://");
 
-        if (g_path_is_absolute (icon_name)) {
-            pix = gdk_pixbuf_new_from_file_at_size (icon_name, w, h, NULL);
-        }
-        else if (g_str_has_prefix (icon_name, "file://")) {
-            gchar *filename = g_filename_from_uri (icon_name, NULL, NULL);
-            if (filename)
-              pix = gdk_pixbuf_new_from_file_at_size (filename, w, h, NULL);
-            g_free (filename);
-        }
-        else {
+        if (is_absolute || is_uri) {
+            GFile *file = is_absolute ? g_file_new_for_path (icon_name) : g_file_new_for_uri (icon_name);
+
+            if (g_file_query_exists (file, NULL) && g_file_query_file_type (file, G_FILE_QUERY_INFO_NONE, NULL) == G_FILE_TYPE_REGULAR) {
+                icon = g_file_icon_new (file);
+            }
+            g_object_unref (file);
+        } else {
             icon = g_themed_icon_new_with_default_fallbacks (icon_name);
-            gtk_image_set_from_gicon (GTK_IMAGE (window->icon), icon, GTK_ICON_SIZE_DIALOG);
-            icon_set = TRUE;
-        }
-
-        if (pix) {
-            gtk_image_set_from_pixbuf (GTK_IMAGE (window->icon), pix);
-            g_object_unref (G_OBJECT (pix));
-            icon_set = TRUE;
         }
     }
 
-    if (icon_set)
+    if (icon != NULL) {
+        gtk_image_set_from_gicon (GTK_IMAGE (window->icon), icon, GTK_ICON_SIZE_DIALOG);
         gtk_widget_show (window->icon_box);
-    else {
+        g_object_unref (icon);
+    } else {
         gtk_image_clear (GTK_IMAGE (window->icon));
         gtk_widget_hide (window->icon_box);
     }
 
-    if (gtk_widget_get_realized (GTK_WIDGET (window)))
-        gtk_widget_queue_draw (GTK_WIDGET (window));
+    if (gtk_widget_get_realized(GTK_WIDGET(window))) {
+        gtk_widget_queue_draw(GTK_WIDGET(window));
+    }
 }
 
 void
 xfce_notify_window_set_icon_pixbuf(XfceNotifyWindow *window,
                                    GdkPixbuf *pixbuf)
 {
-    GdkPixbuf *p_free = NULL;
-
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window)
                      && (!pixbuf || GDK_IS_PIXBUF(pixbuf)));
 
     if(pixbuf) {
-        gint w, h, pw, ph;
+        GdkWindow *for_window = gtk_widget_get_window(GTK_WIDGET(window));
+        gint scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(window));
+        cairo_surface_t *surface;
+        gint w, h, size, pw, ph;
 
         gtk_icon_size_lookup(GTK_ICON_SIZE_DIALOG, &w, &h);
+        size = MIN(w, h) * scale_factor;
         pw = gdk_pixbuf_get_width(pixbuf);
         ph = gdk_pixbuf_get_height(pixbuf);
 
-        if(w > h)
-            w = h;
-        if(pw > w || ph > w) {
+        if(pw > size || ph > size) {
+            GdkPixbuf *pix_scaled;
             gint nw, nh;
 
             if(pw > ph) {
-                nw = w;
-                nh = w * ((gdouble)ph/pw);
+                nw = size;
+                nh = size * ((gdouble)ph/pw);
             } else {
-                nw = w * ((gdouble)pw/ph);
-                nh = w;
+                nw = size * ((gdouble)pw/ph);
+                nh = size;
             }
 
-            pixbuf = p_free = gdk_pixbuf_scale_simple(pixbuf, nw, nh,
-                                                      GDK_INTERP_BILINEAR);
+            pix_scaled = gdk_pixbuf_scale_simple(pixbuf, nw, nh,
+                                                 GDK_INTERP_BILINEAR);
+            surface = gdk_cairo_surface_create_from_pixbuf(pix_scaled, scale_factor, for_window);
+            g_object_unref(pix_scaled);
+        } else {
+            surface = gdk_cairo_surface_create_from_pixbuf(pixbuf, scale_factor, for_window);
         }
+
+        gtk_image_set_from_surface(GTK_IMAGE(window->icon), surface);
+        gtk_widget_show(window->icon_box);
+        cairo_surface_destroy(surface);
+    } else {
+        gtk_image_clear(GTK_IMAGE(window->icon));
+        gtk_widget_hide(window->icon_box);
     }
 
-    gtk_image_set_from_pixbuf(GTK_IMAGE(window->icon), pixbuf);
-
-    if(pixbuf)
-        gtk_widget_show(window->icon_box);
-    else
-        gtk_widget_hide(window->icon_box);
-
-    if(gtk_widget_get_realized(GTK_WIDGET(window)))
+    if(gtk_widget_get_realized(GTK_WIDGET(window))) {
         gtk_widget_queue_draw(GTK_WIDGET(window));
-
-    if(p_free)
-        g_object_unref(G_OBJECT(p_free));
+    }
 }
 
 void
@@ -803,8 +917,17 @@ xfce_notify_window_set_expire_timeout(XfceNotifyWindow *window,
         }
         gtk_widget_set_opacity(GTK_WIDGET(window), window->normal_opacity);
         /* reset the sliding-out window to its original position */
-        if (window->do_slideout && window->original_x >= 0)
-            gtk_window_move (GTK_WINDOW (window), window->original_x, window->original_y);
+        if (window->do_slideout && window->original_x >= 0) {
+#ifdef ENABLE_WAYLAND
+            if (GDK_IS_WAYLAND_DISPLAY(gdk_display_get_default())) {
+                gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_LEFT, window->original_x);
+                gtk_layer_set_margin(GTK_WINDOW(window), GTK_LAYER_SHELL_EDGE_TOP, window->original_y);
+            } else
+#endif
+            {
+                gtk_window_move (GTK_WINDOW (window), window->original_x, window->original_y);
+            }
+        }
 
         xfce_notify_window_start_expiration (window);
     }
@@ -813,8 +936,10 @@ xfce_notify_window_set_expire_timeout(XfceNotifyWindow *window,
 void
 xfce_notify_window_set_actions(XfceNotifyWindow *window,
                                const gchar **actions,
+                               gboolean actions_are_icon_names,
                                GtkCssProvider *css_provider)
 {
+    GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
     gint i;
     GList *children, *l;
 
@@ -836,8 +961,7 @@ xfce_notify_window_set_actions(XfceNotifyWindow *window,
     for(i = 0; actions && actions[i]; i += 2) {
         const gchar *cur_action_id = actions[i];
         const gchar *cur_button_text = actions[i+1];
-        GtkWidget *btn, *lbl;
-        gchar *cur_button_text_escaped;
+        GtkWidget *btn, *img = NULL, *lbl;
         gdouble padding;
 
         if(!cur_button_text || !cur_action_id || !*cur_action_id)
@@ -864,19 +988,35 @@ xfce_notify_window_set_actions(XfceNotifyWindow *window,
                          G_CALLBACK(xfce_notify_window_button_clicked),
                          window);
 
-        cur_button_text_escaped = g_markup_printf_escaped("<span size='small'>%s</span>",
-                                                          cur_button_text);
+        if (actions_are_icon_names && gtk_icon_theme_has_icon(icon_theme, cur_action_id)) {
+            GIcon *icon = g_themed_icon_new_with_default_fallbacks(cur_action_id);
 
-        lbl = gtk_label_new(NULL);
-        gtk_label_set_markup(GTK_LABEL(lbl), cur_button_text_escaped);
-        gtk_label_set_use_markup(GTK_LABEL(lbl), TRUE);
-        gtk_widget_show(lbl);
-        gtk_container_add(GTK_CONTAINER(btn), lbl);
-        gtk_style_context_add_provider (gtk_widget_get_style_context (btn),
-                                        GTK_STYLE_PROVIDER (css_provider),
-                                        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+            img = gtk_image_new_from_gicon(icon, GTK_ICON_SIZE_BUTTON);
+            gtk_widget_set_tooltip_text(img, cur_button_text);
+            atk_object_set_description(gtk_widget_get_accessible(img), cur_button_text);
+            gtk_widget_show(img);
+            gtk_container_add(GTK_CONTAINER(btn), img);
 
-        g_free(cur_button_text_escaped);
+            gtk_widget_set_tooltip_text(btn, cur_button_text);
+
+            g_object_unref(icon);
+        }
+
+        if (img == NULL) {
+            gchar *cur_button_text_escaped = g_markup_printf_escaped("<span size='small'>%s</span>",
+                                                                     cur_button_text);
+
+            lbl = gtk_label_new(NULL);
+            gtk_label_set_markup(GTK_LABEL(lbl), cur_button_text_escaped);
+            gtk_label_set_use_markup(GTK_LABEL(lbl), TRUE);
+            gtk_widget_show(lbl);
+            gtk_container_add(GTK_CONTAINER(btn), lbl);
+            gtk_style_context_add_provider (gtk_widget_get_style_context (btn),
+                                            GTK_STYLE_PROVIDER (css_provider),
+                                            GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+            g_free(cur_button_text_escaped);
+        }
     }
 
     if(gtk_widget_get_realized(GTK_WIDGET(window)))
@@ -960,8 +1100,6 @@ xfce_notify_window_set_gauge_value(XfceNotifyWindow *window,
     else if(value < 0)
         value = 0;
 
-    gtk_widget_hide(window->summary);
-    gtk_widget_hide(window->body);
     gtk_widget_hide(window->button_box);
 
     if(!window->gauge) {
@@ -1040,6 +1178,21 @@ void xfce_notify_window_set_notify_location(XfceNotifyWindow *window,
     window->notify_location = notify_location;
 }
 
+#ifdef ENABLE_SOUND
+void
+xfce_notify_window_set_sound_props(XfceNotifyWindow *window,
+                                   ca_proplist *props)
+{
+    if (window->sound_props != props) {
+        if (window->sound_props != NULL) {
+            ca_proplist_destroy(window->sound_props);
+        }
+
+        window->sound_props = props;
+    }
+}
+#endif
+
 void
 xfce_notify_window_closed(XfceNotifyWindow *window,
                           XfceNotifyCloseReason reason)
